/*
 * armboot_main.c
 * ARM ROM boot loader main file.
 *
 * Copyright (C) 2009 Texas Instruments Incorporated - http://www.ti.com/
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation version 2.
 *
 * This program is distributed "as is" WITHOUT ANY WARRANTY of any
 * kind, whether express or implied; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#include <bootloader.h>
#include <uart.h>

Uint32 gEntryPoint;
NAND_BOOT gNandBoot;				 /* structure for holding details about UBL stored in NAND */
extern void platformsetup(void);
Uint32 DM700_UART0OpenDevice();
void UARTSWReset();

/* ------------------------------------------------------------------------ *
 *  Device System Controller                                                *
 * ------------------------------------------------------------------------ */
#define SYS_BASE                0x01c40000
#define SYS_PINMUX0             *( volatile Uint32* )( SYS_BASE + 0x0 )
#define SYS_PINMUX1             *( volatile Uint32* )( SYS_BASE + 0x4 )
#define SYS_DSPBOOTADDR         *( volatile Uint32* )( SYS_BASE + 0x8 )
#define SYS_SUSPSRC             *( volatile Uint32* )( SYS_BASE + 0xc )
#define SYS_BOOTSTAT            *( volatile Uint32* )( SYS_BASE + 0x10 )
#define SYS_BOOTCFG             *( volatile Uint32* )( SYS_BASE + 0x14 )
#define SYS_SMTREFLEX           *( volatile Uint32* )( SYS_BASE + 0x18 )
#define SYS_ARMBOOT             *( volatile Uint32* )( SYS_BASE + 0x24 )
#define SYS_JTAG_ID             *( volatile Uint32* )( SYS_BASE + 0x28 )
#define SYS_HDVICPBOOT          *( volatile Uint32* )( SYS_BASE + 0x2c )
#define SYS_HPICTL              *( volatile Uint32* )( SYS_BASE + 0x30 )
#define SYS_USBCTL              *( volatile Uint32* )( SYS_BASE + 0x34 )
#define SYS_VIDCLKCTL           *( volatile Uint32* )( SYS_BASE + 0x38 )
#define SYS_MSTPRI0             *( volatile Uint32* )( SYS_BASE + 0x3c )
#define SYS_MSTPRI1             *( volatile Uint32* )( SYS_BASE + 0x40 )
#define SYS_MSTPRI2             *( volatile Uint32* )( SYS_BASE + 0x44 )
#define SYS_VDD3P3VPWDN         *( volatile Uint32* )( SYS_BASE + 0x48 )
#define SYS_TSIFCTL             *( volatile Uint32* )( SYS_BASE + 0x50 )
#define SYS_PWMCTL              *( volatile Uint32* )( SYS_BASE + 0x54 )
#define SYS_ETCCFG              *( volatile Uint32* )( SYS_BASE + 0x58 )
#define SYS_CLKCTL              *( volatile Uint32* )( SYS_BASE + 0x5c )
#define SYS_DSPINT              *( volatile Uint32* )( SYS_BASE + 0x60 )
#define SYS_DSPINTSET           *( volatile Uint32* )( SYS_BASE + 0x64 )
#define SYS_DSPINTCLR           *( volatile Uint32* )( SYS_BASE + 0x68 )
#define SYS_VSCLKDIS            *( volatile Uint32* )( SYS_BASE + 0x6c )
#define SYS_ARMINT              *( volatile Uint32* )( SYS_BASE + 0x70 )
#define SYS_ARMINTSET           *( volatile Uint32* )( SYS_BASE + 0x74 )
#define SYS_ARMINTCLR           *( volatile Uint32* )( SYS_BASE + 0x78 )
#define SYS_ARMWAIT             *( volatile Uint32* )( SYS_BASE + 0x7c ) 

#define CSL_SYS_VDD3P3VPWDN_UR0MDM_MASK (0x00000020u)
#define CSL_SYS_VDD3P3VPWDN_UR0MDM_SHIFT (0x00000005u)

#define CSL_SYS_VDD3P3VPWDN_UR0DF_MASK (0x00000010u)
#define CSL_SYS_VDD3P3VPWDN_UR0DF_SHIFT (0x00000004u)

Uint32 LPSCEnable(Uint8 PdNum, Uint8 LPSCNUM) 
{
	volatile Uint32 count;
 
  	/* NEXT=0x3, Enable LPSC Module N */   
  	CSL_PSC_0_REGS->MDCTL[LPSCNUM] = (CSL_PSC_0_REGS->MDCTL[LPSCNUM]&0xFFFFFFE0) | (0x3);

	/* Power Domain 0 ON	*/
	CSL_PSC_0_REGS->PDCTL[PdNum] |= 0x1;
	CSL_PSC_0_REGS->PTCMD = (1 << PdNum);	

	/* Wait for GOSTAT = NO TRANSITION from PTSTAT */
	count = TIMEOUT_COUNT;
	while( (((CSL_PSC_0_REGS->PTSTAT >> PdNum) & 0x00000001) != 0) && (count > 0)){
		count--;
	}

	if(count == 0)
		return	E_TIMEOUT;

  	/* Wait for MODSTAT = ENABLE from LPSC */
    count = TIMEOUT_COUNT;
    while( ((CSL_PSC_0_REGS->MDSTAT[LPSCNUM] & 0x0000001F) != 0x3) && (count > 0)){
   		count --;
   	}

   	if (count == 0)
		return E_TIMEOUT;

	return E_PASS;
}

void ICache1()
{
        asm("   STMFD   SP!, {R0-R1}");
        asm("	MOV     R0, #0");
        asm("	MCR     P15, #0, R0, C7, C5, #0");
        asm("	NOP");
        asm("	NOP");
        asm("	NOP");
        asm("	NOP");
	asm("	MRC   P15, #0, R1, C1, C0, #0");
 	asm("	MOV   R0,#0x1000");
}

void ICache2()
{
        asm("	NOP");
   	asm("	NOP");
	asm("	MCR   P15, #0, R1, C1, C0, #0");
   	asm("	NOP");
   	asm("	NOP");
   	asm("   LDMFD  SP!, {R0-R1}");
}
void EnableICache()
{
        /* Flush I-Cache */
	ICache1();
	asm("	ORR   R1, R1, R0");
	ICache2();
}

void DisableICache()
{
        /* Flush I-Cache */
	ICache1();
	asm("	BIC   R1, R1, R0");
   	ICache2();
}

void waitloop (Uint32 count)
{
        volatile Uint32 counter;
        for (counter=0; counter<count; counter++) {
                asm("\tnop");
        }
}

void init_vect()
{
#ifdef UBL_USE_NOR_BOOT
	volatile Uint32 *ivect = (volatile Uint32 *)0x10000;
	*ivect++ = 0xEAFFFFFE;  /* Reset @ 0x00*/
#else
	volatile Uint32 *ivect = (volatile Uint32 *)0x10004;
#endif
	*ivect++ = 0xEAFFFFFE;  /* Undefined Address @ 0x04 */
	*ivect++ = 0xEAFFFFFE;  /* Software Interrupt @0x08 */
	*ivect++ = 0xEAFFFFFE;  /* Pre-Fetch Abort @ 0x0C */
	*ivect++ = 0xEAFFFFFE;  /* Data Abort @ 0x10 */
	*ivect++ = 0xEAFFFFFE;  /* Reserved @ 0x14 */
	*ivect++ = 0xEAFFFFFE;  /* IRQ @ 0x18 */
	*ivect   = 0xEAFFFFFE;	/* FIQ @ 0x1C */
}

#define 	DDR_TEST_FLAG		(0xAAAAAAAA)
Int testDDR (void)
{
        volatile Uint32	count = 0;
	volatile Uint32	*ptr = (volatile Uint32	*) 0x8E000000;

	UARTSendData(21,(Uint8 *) "\r\nStarting RAM Test\r\n");

	*ptr = DDR_TEST_FLAG;
	if(*ptr != DDR_TEST_FLAG) {
		UARTSendData(19,(Uint8 *) "\r\nRAM Test Failed\r\n");
		return -1;
	}

	for(count = 0; count < 0x02000000; count ++) {
		*ptr = 0;
		ptr++;
	}

	UARTSendData(19,(Uint8 *) "\r\nRAM Test Passed\r\n");

	return 0;
}

#define PSC_MDCTL_DSP       *( unsigned int* )( 0x01c41a00 + ( 4 * 1 ) )

Int main(void)
{
	volatile BootMode	mode;
	Uint16 k;

	init_vect();

	/* Put DSP in reset */
	/*PSC_MDCTL_DSP &= 0x0FF;*/

	/* Clear all interrupts (FIQ and IRQ) and disable all interrupts */
	CSL_INTC1_0_REGS->INTCTL = 0x0; /* settinf IDMODE = 0 for immediate disable */
	CSL_INTC1_0_REGS->EINT0 = 0x0; /* 0-31 INT */
	CSL_INTC1_0_REGS->EINT1 = 0x0; /* 32-63 INT */

	DM700_UART0OpenDevice();
	*(volatile unsigned int *)PINMUX1	= 0;		// UART0 function with modem control.
	LPSCEnable(PD0, LPSC_UART0);
	UARTSWReset();

	/* Initialize the UART */
	UARTModeInit();

	CSL_UART_0_REGS->IIR	|=	0x07;	//Enable FIFO
	CSL_UART_0_REGS->IIR	 =	0;
	while(CSL_UART_0_REGS->LSR & 1)		//change
	{
		k=CSL_UART_0_REGS->RHR;			//Clear Read FIFO Buf.
	}

	/* Send some debug Messages */
	UARTSendData(27,(Uint8 *) "\r\nBooting PSP Boot Loader\r\n");

#ifdef UBL_USE_NAND_BOOT
	mode = NON_SECURE_NAND;
#endif

#ifdef UBL_USE_NOR_BOOT
	mode = NON_SECURE_NOR;
#endif

#ifdef UBL_USE_PCI_BOOT
	mode = NON_SECURE_PCI;
#endif

#ifdef UBL_USE_UART_PROG
	/* Check for a sequence */
	if(E_PASS == UARTCheckSequence(2,(Uint8 *)"  ")) {
		UARTSendData(28,(Uint8 *) "\r\nStarting UART Programmer\r\n");
		mode = NON_SECURE_UART_PRG;
	} else {
		mode = NON_SECURE_UART;
	}
#endif

#if UBL_INCLUDE_RAM_TEST
REPEAT_DDR_INIT:
#endif /* UBL_INCLUDE_RAM_TEST */

	/* Initialize DDR */
	platformsetup();

#if UBL_INCLUDE_RAM_TEST
	if(testDDR() != 0) {
		goto REPEAT_DDR_INIT;
	}
#endif /* UBL_INCLUDE_RAM_TEST */

	/* Select Boot Mode */
	switch(mode) {
#ifdef UBL_USE_NAND_BOOT
		case NON_SECURE_NAND:
			if(NANDInit() == E_PASS) {
				/* copy user boot loader from NAND to DDRAM */
				if (NANDCopy() != E_PASS) {
					UARTSendData(36,(Uint8 *) "\r\nNAND Boot failed. Starting UART\n\r\r");
					UARTBoot();
				}
			}
			break;
#endif
#ifdef UBL_USE_NOR_BOOT
		case NON_SECURE_NOR:
			if (NORCopy() != E_PASS) {
				UARTSendData(35,(Uint8 *) "\r\nNOR Boot failed. Starting UART\n\r\r");
				UARTBoot();
			}
			break;
#endif
#ifdef UBL_USE_UART_BOOT
		case NON_SECURE_UART:
			UARTBoot();
			break;
#endif
#ifdef UBL_USE_UART_PROG
		case NON_SECURE_UART_PRG:
			UartProgrammer();
			break;
#endif
#ifdef UBL_USE_PCI_BOOT
		case NON_SECURE_PCI:
			PCIBoot();
			break;
#endif
		default:
			UARTBoot();
			break;
	}

	UARTSendData(26,(Uint8*)"\r\nBooting Application @ 0x");
	UARTSendInt(gEntryPoint);
	UARTSendData(2,(Uint8*)"\r\n");

	/* Disabling Timers */
	CSL_TMR_0_REGS->TCR = 0;
	CSL_TMR_1_REGS->TCR = 0;

	/* Removing Reset of Watch Dog Timer
	 * as its causing ARM Reset @ Turbo Mode */
	// CSL_TMR_2_REGS->TCR = 0;

	return E_PASS;
}

Uint32 DM700_UART0OpenDevice()
{
	SYS_CLKCTL	&=	0xFCFFFFFF;
	CSL_FINS(SYS_VDD3P3VPWDN, SYS_VDD3P3VPWDN_UR0MDM, 0);
	CSL_FINS(SYS_VDD3P3VPWDN, SYS_VDD3P3VPWDN_UR0DF, 0);

	return 0;
}

void UARTSWReset()
{
	//UART 0 SW RESET
	CSL_UART_0_REGS->LCR	=	0xBF;	// First write to the LCR register
	//EFR in this mode occupy same address as IIR in overlay structure
	CSL_UART_0_REGS->IIR	|=	0x10;	// When LCR = 0xBF enable the enhanced feature register

	CSL_UART_0_REGS->LCR	&=	0x7F;	// Here, access to IER and MCR is allowed.
	CSL_UART_0_REGS->IER	=	0;		// Disable interrupt
	CSL_UART_0_REGS->MCR	=	0;		// Force control signals inactive
	CSL_UART_0_REGS->LCR	&=	0xBF;	// Here, UART breaks are removed
	CSL_UART_0_REGS->RHR	=	0x07;	/// Here, UART is in reset or disabled
	return;
}

