/*
 * Copyright 2007 by Spectrum Digital Incorporated.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation version 2.
 *
 * This program is distributed "as is" WITHOUT ANY WARRANTY of any
 * kind, whether express or implied; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

/*
 *  DDR Setup
 *
 */

#include "davincihd_ddr.h"
#include "davincihd_psc.h"

/* ------------------------------------------------------------------------ *
 *                                                                          *
 *  DAVINCIHD_DDR_init( freq )                                              *
 *      Setup the DDR2 & DDR2 PHY to 300 MHz                                *
 *                                                                          *
 * ------------------------------------------------------------------------ */
Int16 DAVINCIHD_DDR_init( Int32 freq )
{
    Uint32 davincihd_ddr_init_fn;
    Uint32 refresh_rate;
    static Uint32 dummy_read;

    /*
     *  Check 0 - Check to see if we are running from DDR
     *            If so, then its a bad idea to run this DDR setup
     */
    davincihd_ddr_init_fn = ( Uint32 )&DAVINCIHD_DDR_init;
    if ( ( davincihd_ddr_init_fn >= ( DDR_BASE ) )
      && ( davincihd_ddr_init_fn <= ( DDR_BASE + DDR_SIZE ) ) )
        return -1;

    /*
     *  Step 0 - Stop all DDR accesses.
     */

    /*
     *  Step 1 - Setup PLL2
     *  Step 2 - Enable DDR2 PHY
     */
    DAVINCIHD_PSC_changeState( LPSC_DDR2_EMIF, PSC_ENABLE );

    /*
     *  Step 3 - VTP Calibration
     */
    DDR_VTPIOCR = 0x00000003;  // Set VTP_RECAL bit.
    DDR_VTPIOCR = 0x00000002;  // Toggle VTP_RECAL bit.
	
    // Wait for completion of VTP calibration and DDLL ready.
    while(!(DDR_SDRSTAT &  0x00000004) == 0x04); // Wait for phy_dll_ready/PHYRDY bit set.

    /*
     *  Step 4 - Soft Reset ( SYNCRESET followed by ENABLE ) of DDR2 PHY
     */
    DAVINCIHD_PSC_changeState( LPSC_DDR2_EMIF, PSC_SYNCRESET );
    DAVINCIHD_PSC_changeState( LPSC_DDR2_EMIF, PSC_ENABLE );

    /*
     *  Step 5 - Wait for 10msec
     */
    DAVINCIHD_waitmsec( 10 );

    /*
     *  Step 6 - DDR2 Initialization
     */
    DDR_DDRPHYCR = 0x00008AC7;          // DLL powered, ReadLatency=7
    DDR_SDBCR    = 0x08D78A32;          // DDR Bank: 32-bit bus, CAS=5,
                                        // 8 banks, 1024-word pg, unlocl
    DDR_SDTIMR   = 0x4B245C12;
    DDR_SDTIMR2  = 0x3B2BC742;
    DDR_SDRCR    = 0x90D;               // Refresh Control [ 7.8 us * 297 MHz ]

    DDR_SDBCR    = 0x08570A32;          // Lock values

    /*
     *  Step 7 - Dummy Read
     */
    dummy_read = DDR_BASE;
    dummy_read = *( volatile Uint32* )dummy_read;

    return 0;
}
