/*
 * Copyright 2007 by Spectrum Digital Incorporated.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation version 2.
 *
 * This program is distributed "as is" WITHOUT ANY WARRANTY of any
 * kind, whether express or implied; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

/*
 *  NAND Flash interface
 *
 */

#include "davincihd_nandflash.h"

/*
 *  NAND Organization
 */
extern NAND_ORGANIZATION nand_org;

extern Int32 nand_rw_timeout;
/* ------------------------------------------------------------------------ *
 *                                                                          *
 *  _NAND_write8( tx, store_spare, use_ecc )                                *
 *      Read a page from NAND                                               *
 *                                                                          *
 *      Inputs:                                                             *
 *          rx          = address of rx buffer [ any address ]              *
 *          use_ecc     = use ECC?                                          *
 *                                                                          *
 *      Return:                                                             *
 *          =0  pass                                                        *
 *          >0  failed - bad pages                                          *
 *          -1  timeout                                                     *
 *                                                                          *
 * ------------------------------------------------------------------------ */
Uint32 _NAND_write8( Uint32 tx, Uint32 use_ecc )
{
    Int16 i;
	Int16 j,k;
    volatile Uint8* tx8 = ( volatile Uint8* )tx;
	Uint32 store_ecc[4];


    /* Spare + ECC */
    Uint32 computed_ecc;

    /* Start ECC */
    if ( use_ecc )
        _NAND_startECC( );

    /* Write MAIN array */
    i = nand_org.page_size;
    while ( i!=0 )
   	{
        NAND_DATA = *tx8++;
		i--;
		
		if(i%512==0)
			{
			    /* Compute ECC */
		    	
			    if ( use_ecc )
			    {

			        computed_ecc = _NAND_readECC( );
					_NAND_startECC( );
			        computed_ecc &= 0x0FFF0FFF;				
					store_ecc[i/512]=computed_ecc;					
			 	}
			}
		
		
        
    }


		k=8;
		/* Write Spare array*/
		while(k--)
			NAND_DATA = 0xFF;

	for(j=3;j>=0;j--)	
	{
	
		computed_ecc= store_ecc[j];
					
       	NAND_DATA = ( computed_ecc >> 24 );
	    NAND_DATA = ( computed_ecc >> 16 );
	    NAND_DATA = ( computed_ecc >> 8 );
		NAND_DATA = ( computed_ecc );	

		
		k=12;
		/* Write Spare array*/
		while(k--)
			NAND_DATA = 0xFF;
	}

    return 0;
}

/* ------------------------------------------------------------------------ *
 *                                                                          *
 *  _NAND_writePage( tx, block_num, start_page, page_count )                           *
 *                                                                          *
 *      Program NAND Flash.                                                 *
 *          tx         = address of tx buffer                               *
 *          start_page = start page                                         *
 *          page_count = # of pages                                         *
 *                                                                          *
 *      Returns:    0   pass                                                *
 *                  1+  failed - bad pages                                  *
 *                 -1   timeout                                             *
 *                                                                          *
 * ------------------------------------------------------------------------ */
Int16 _NAND_writePage( Uint32 tx, Uint32 block_num, Uint32 start_page, Uint32 page_count )
{
    Uint32 i;
    Uint32 page_addr;
    Int16 retcode = 0;

	start_page = (block_num * NAND_PAGES_PER_BLOCK) + start_page;

    for ( i = start_page ; i < start_page + page_count ; i++ )
    {
        /* Fix for large page addressing */
        page_addr = i * nand_org.page_addr_incr;

        /* Write a single page */
        _NAND_CMD( CMD_PROGRAM );
        _NAND_PAGE_ADDR( page_addr );

		printf ("Writing to page %d\n", start_page);

        /* Write page in 8 bit mode */
        if ( nand_org.bus_width == 8 )
            _NAND_write8( tx, 1 ); /*1 to enable ECC,0 to disable*/

        /* Confirm Program */
        _NAND_CMD( CMD_PROGRAM_CONFIRM );

		_waitusec( 500 );

        if ( _NAND_busywait( nand_rw_timeout ) )
            return NAND_ERR_TIMEOUT;

        /* Check Status */
        _NAND_CMD( CMD_STATUS );
        if ( NAND_DATA & CMD_STATUS_ERROR )
        {
            retcode = -1;
        }
    }

    return retcode;
}
